#include <fstream.h>
#include <cmath>
#include <cstdlib>
#include <string>
#include <ctime>

//Number of grid cells per side in study area.
  const int CELLS_PER_SIDE = 50;

//Value of pi used to convert between radians and degrees.
  const double PI = 3.14159265358979;

//Categorical class that distribution occurs within.
  const int CatType = 42;

    void seed(int, double&, double, double&, double);
    void sq_survey(int, int, double, double, double, double, double, int, 
                   double, int [], int [], int[],
                   double [][CELLS_PER_SIDE * CELLS_PER_SIDE], int&, int&, 
                   double, double);
    void strat_survey(int, int, double, int, double, int, double, int [], 
                      int [], int[], int[],
                      double [][CELLS_PER_SIDE * CELLS_PER_SIDE], int&);
    void adapt_survey(int, int, int, double, double, double, double, 
                      int [], int [], int[],
                      double [][CELLS_PER_SIDE * CELLS_PER_SIDE], 
                      int&, double);
    void sys_survey(int, int, double, double, double, int, double, int[],
                    int [], int [], double [][CELLS_PER_SIDE * CELLS_PER_SIDE],
                    int&, double, double, double);
    void rec_survey(int, int, double, double, double, int, double, double,
                    double, int, double, int [], int [], int[],
                    double [][CELLS_PER_SIDE * CELLS_PER_SIDE], int&, int&,
                    double, double);
    void samples(int, double&, double, int, double&, double, int);
    void observe_spp(double, double, double, double, int, double, int [], 
                     int [], int[], int&);
    void rec_xy(double [][CELLS_PER_SIDE * CELLS_PER_SIDE], double, double, 
                int&);
    void create_files(double, int [], int [], int,
                      double [][CELLS_PER_SIDE * CELLS_PER_SIDE], int []);
    void c_prompt(int);
    double quadrat_area(int, double, int);
    int cell_num(double, double, int, double, double);
    int randnum(int, int);

int main(int argc, char * argv[])
/**********************************************************************************************
* AUTHOR: Gordon Reese, FOR: CSU Graduate Research,  DATE:06/05/01 - 10/01/02                 *
***********************************************************************************************
* Program Description:                                                                        *
*     Surveys a square landscape (divided into grid cells) using a chosen surveying design    *
* (see parameters below).  Survey locations are compared to information from an external file *
* containing distribution data, (1,0). Surveys produce presences only where the surveyed      *
* grid cell is part of the distribution and when a computer-generated uniform (0,1) variate   *
* is less than or equal to the detection probability. Searched locations and the outcome of   *
* surveying are recorded as effort.g and respnse.y, respectively; files are formatted for use *
* with autologistic regression.  Program also saves x,y locations in XY.txt file.             *
*                                                                                             *
* Program is currently set to use a categorical covariate with specific class codes. If a     *
* different number (3) of class codes or different class code values are used, modify lines   *
* 14, 460, and 960 - 1050 in surveys.c.  If no categorical covariate is used, do not use the  *
* stratified survey design, and eliminate lines 960 - 1050.                                   *
*                                                                                             *
* Input line should read "survey  LL_X_coord  LL_Y_coord  Cell_size  Num_categorical_codes    * 
*                         SppDist.txt  Detect_prob  Sample_size  Survey_design(r,s,a,y,q,e)   * 
*                         Prop_sampled  Num_quadrats  Rectang_width  CatCodes.txt"            *
*                                                                                             *
* SppDist.txt and CatCodes.txt files should be formatted with header line = number of cells   *
*   and spaces between values or one cell value per line.  Currently set for integer type.    *
*                                                                                             *
* Tests run with ALR execution line:                                                          *
* "autologit response.y covariates.txt effort.g 20000 1000 20 1 2 1 0 0 0 0 output_directory" *
*   Arguments:                                                                                *
*     1) autologit - executable autologistic regression (ALR) program                         *
*     2) response.y - file of response data (0 = absence; 1 = presence)                       *
*     3) covariates.txt - file of covariate data (e.g., environmental variables; 1 cell/line) *
*     4) effort.g - file of surveyed locations (0 = surveyed cell; 1 = unsurveyed cell)       *
*     5) 20000 - number of iterations for autologistic regression program                     *
*     6) 1000 - number of iterations for burn-in (initialization; not used in calculations)   *
*     7) 20 - the variance for the normal prior distributions of theta-sub k, k=0, ..., p     *
*     8) 1 - the 1st parameter in the gamma prior for beta where the mean = (this input *     *
*            next input) and the variance = (this input * next input squared)                 *
*     9) 2 - the 2nd parameter in the gamma prior for beta                                    *
*    10) 1 - the starting value for beta                                                      *
*    11) 0 0 0 0 - the starting values for theta-sub k, k=0, ..., p; one for each of          *
*                  number of covariates in covariates.txt (dummies for categorical) + 1       *
*    12) output_directory - directory for the output files                                    *
*                                                                                             *
***********************************************************************************************
* Important Consts/Variables:                                                                 *
* (char) survey_design      - holds the switch for survey design                              *
* (constant) CELLS_PER_SIDE - # of grid cells to a side in study extent                       *
* (constant) PI             - used for conversions between radians & degrees                  *
* (double) ll_x_coord       - longitude of lower left corner of study area                    *
* (double) ll_y_coord       - latitude of lower left corner of study area                     *
* (double) tr_x_coord       - longitude of top right corner of study area                     *
* (double) tr_y_coord       - latitude of top right corner of study area                      *
* (double) total_area       - total area for the grid being surveyed                          *
* (double) prop_sampled     - proportion of study area user wants to survey                   *
* (double) detect_prob      - probability of detecting a surveyed distribution                *
* (double) ea_quadrat_area  - size (meters sq.) of each quadrat in study area                 *
* (double) sq_side_length   - the length(m) for each side of square quadrats                  *
* (double) survey_x_coord   - longitudinal co-ordinate                                        *
* (double) survey_y_coord   - latitudinal co-ordinate                                         *
* (double) sample_x         - random longitude location to sample in survey                   *
* (double) sample_y         - random latitude locatoin to sample in survey                    *
* (double) biased_width     - the length (x 2) of minor axis in rectangular survey            *
* (double) biased_length    - the length(m) for side runing parallel to major axis            *
* (double) xy_coord[][]     - records co-ordinates of surveyed locations                      *
* (double) sys_area         - area represented by indiv. systematic quadrats                  * 
* (double) sys_side_length  - the length(m) for a side of previous variable                   *
* (int) cell_size           - the length(m) of each of the grid cells                         *
* (int) num_quadrats        - # of quadrats in which to divide the survey area                *
* (int) num_samples         - # of locations to be surveyed                                   *
* (int) num_samples_count   - # of locations to be surveyed                                   *
* (int) num_catcodes        - # of unique categorical classes                                 *
* (int) i, j, x, y          - incrementors                                                    *
* (int) trash               - for discarding the first line of text files                     *
* (int) effort[]            - record of surveyed cells                                        *
* (int) response[]          - record of survey results                                        *
* (int) sppdist[]           - holds distribution information                                  *
* (int) catcodes[]          - holds categorical class information                             *
* (int) xy_count            - counter to record x,y coordinates of samples                    *
* (int) random_num1 & 2     - random # between defined high & low integers                    *
* (int) biased_angle        - random orientation angle of rectangular quadrat                 *
**********************************************************************************************/
{
    if (argc != 13) 
    {
        cout << endl;
        cout << "Usage: survey  LL_X_coord  LL_Y_coord  Cell_size  Num_cat_codes \n";
        cout << "       SppDist.txt  Detect_prob  Sample_size  Survey_design(r,s,a,y,q,e) \n";
        cout << "       Prop_sampled  Num_quadrats  Rectang_width \n";
        cout << "       CatCodes.txt \n";
        cout << endl;
        cout << " set Detection probability (Detect_prob) between 0, 1 \n";
        cout << " set Sample_size to real number \n";
        cout << endl;
        cout << "Survey designs: \n";
        cout << " R = random      S = stratified  A = adaptive\n";
        cout << " Y = systematic  Q = square      E = rectangular\n";
        cout << endl;
        cout << "Parameters used in this study for survey designs          \n";
        cout << " (where NA, fill with any #)                              \n";
        cout << " ________________________________________________________ \n";
        cout << " Survey_design  Prop_sampled  Num_quadrats  Rectang_width \n";
        cout << " ________________________________________________________ \n";
        cout << "    Random            1.0            1             NA \n";
        cout << "  Stratified           NA           NA             NA \n";
        cout << "   Adaptive            NA           NA             NA \n";
        cout << "  Systematic          0.2           18             NA \n";
        cout << "    Square            0.2           18             NA \n";
        cout << "  Rectangular         0.2           18             50 \n";
        cout << endl;
        cout << endl;
        cout << "CatCodes.txt should be a file containing header (# cells) \n";
        cout << " and categorical class codes. \n";
        cout << endl;
        cout << " Will be used to determine if every class has at least 3 presences, \n";
        cout << "  if no, will introduce the difference. \n";
        cout << endl;
        cout << " *program is set to survey a 50x50 grid (line #8) \n";
        cout << " *program is set to use 1 categorical variable with 3 classes with \n";
        cout << "   specific codes, see lines 14 & 460 and 'create_files' function to change\n";
        cout << endl;
        cout << "  Lines 960-1050 of 'create_files' can be deleted \n";
        cout << "   when no categorical variable is used as a covariate \n";
        cout << endl;
        exit(1);
    }

    double total_area, tr_x_coord, tr_y_coord, ea_quadrat_area;
    double sq_side_length, survey_x_coord, survey_y_coord;
    double biased_length;
    double xy_coord[2][CELLS_PER_SIDE * CELLS_PER_SIDE];
    double sys_area, sys_side_length;
    ifstream in_stream;
    int i, x, y;
    int trash, xy_count, biased_angle, num_samples_count;
    int effort[CELLS_PER_SIDE * CELLS_PER_SIDE];
    int response[CELLS_PER_SIDE * CELLS_PER_SIDE];
    int sppdist[CELLS_PER_SIDE * CELLS_PER_SIDE];
    int catcodes[CELLS_PER_SIDE * CELLS_PER_SIDE];
    ofstream out_stream;

    double ll_x_coord = atof(argv[1]);
    double ll_y_coord = atof(argv[2]);
    int cell_size = atoi(argv[3]);
    int num_catcodes = atoi(argv[4]);
    double detect_prob = atof(argv[6]);
    int num_samples = atoi(argv[7]);
    char survey_design[2];
    strcpy(survey_design, argv[8]);
    double prop_sampled = atof(argv[9]);
    int num_quadrats = atoi(argv[10]);
    double biased_width = atof(argv[11]);

    {
        //Opens file containing species distribution info (file header = # cells).
        in_stream.open(argv[5]);
        if (in_stream.fail())
        {
            cout << "Input file " << argv[5] << "opening failed.\n";
            exit(1);
        }

        //Discards header.
        in_stream >> trash;

        //Loads species distribution information into array.
        for (x = 0; x < (CELLS_PER_SIDE * CELLS_PER_SIDE); x++)
        {
            in_stream >> sppdist[x];
        }//end for

        in_stream.close();
    }

    {
        //Opens file containing categorical variable information (file header = # cells).
        in_stream.open(argv[12]);
        if (in_stream.fail())
        {
            cout << "Input file " << argv[12] << "opening failed.\n";
            exit(1);
        }

        //Discards header.
        in_stream >> trash;

        //Loads categorical variable information into array.
        for (x = 0; x < (CELLS_PER_SIDE * CELLS_PER_SIDE); x++)
        {
            in_stream >> catcodes[x];
        }//end for

        in_stream.close();

    }

    //Calculates area of the study extent.
    total_area = pow(double(CELLS_PER_SIDE * cell_size), 2.0);

    //Calculates co-ordinates for the top-right corner of study extent.
    tr_x_coord = ll_x_coord + double(CELLS_PER_SIDE * cell_size);
    tr_y_coord = ll_y_coord + double(CELLS_PER_SIDE * cell_size);

    //Calculates dimensions for surveying designs that use quadrats.
    switch (survey_design[0])
    {
        case 'r':
        case 'R':
        case 'y':
        case 'Y':
        case 'q':
        case 'Q':
        case 'e':
        case 'E':
            //Calculates the area surveyed by each quadrat.
            ea_quadrat_area = quadrat_area(cell_size, prop_sampled, num_quadrats);

            //Calculates the length of a side of square quadrats.
            sq_side_length = sqrt(ea_quadrat_area);

            //Calculates the area in which systematic quadrats will
            //be generated; such that separation is equidistant in
            //x and y directions.
            sys_area = total_area / num_quadrats;

            //Calculates the length of a side for previous area.
            sys_side_length = sqrt(sys_area);

            break;
    }//end switch

    //Produces a presence/absence dataset and records data in file format
    //for autologistic regression (i.e., autologit.exe).
    //Indented for readability.
    {
        //Sets all array cells to 'null' equivalent.
        for (i = 0; i < (CELLS_PER_SIDE * CELLS_PER_SIDE); i++)
        {
            response[i] = 0;
            effort[i] = 1;
        }

        //Aids recording of the x,y sample location co-ordinates.
        xy_count = 0;

        //Tracks the number of locations sampled.
        num_samples_count = num_samples;

        //Selects appropriate surveying algorithm.
        switch (survey_design[0])
        {
            //Random square quadrat survey designs;
            //includes procedure for surveying entire study extent.
            case 'r':
            case 'R':
            case 'q':
            case 'Q':

                //Random, entire study extent survey design.
                if ((prop_sampled == 1.0) && (num_quadrats == 1))
                {

                    //Calculates co-ordinates for the center of quadrat 
                    //(i.e., entire study extent).
                    survey_x_coord = ll_x_coord + (double(CELLS_PER_SIDE * cell_size) / 2.0);
                    survey_y_coord = ll_y_coord + (double(CELLS_PER_SIDE * cell_size) / 2.0);

                    sq_survey(num_samples, num_quadrats, sq_side_length, survey_x_coord,
                              survey_y_coord, ll_x_coord, ll_y_coord, cell_size,
                              detect_prob, response, effort, sppdist, xy_coord, xy_count,
                              num_samples_count, tr_x_coord, tr_y_coord);

                }//end if
                //Random square quadrats survey design.
                else
                {
                    while (num_samples_count > 0)
                    {
                        seed(cell_size, survey_x_coord, ll_x_coord, survey_y_coord,
                             ll_y_coord);

                        sq_survey(num_samples, num_quadrats, sq_side_length,
                                  survey_x_coord, survey_y_coord, ll_x_coord, ll_y_coord,
                                  cell_size, detect_prob, response, effort, sppdist, xy_coord,
                                  xy_count, num_samples_count, tr_x_coord, tr_y_coord);
                    }//end while (num_samples_count > 0)
                }//end else
                break;
            //Stratified survey design.
            case 's':
            case 'S':
                strat_survey(num_catcodes, num_samples_count, ll_x_coord, cell_size,
                             ll_y_coord, num_samples, detect_prob, response, effort,
                             catcodes, sppdist, xy_coord, xy_count);
                break;
            //Adaptive survey design.
            case 'a':
            case 'A':
                adapt_survey(num_samples, num_samples_count, cell_size,
                             ll_x_coord, ll_y_coord, tr_y_coord, detect_prob, response,
                             effort, sppdist, xy_coord, xy_count, tr_x_coord);
                break;
            //Systematic square quadrat survey design.
            case 'y':
            case 'Y':
                sys_survey(num_samples_count, num_quadrats, sq_side_length, 
                           ll_x_coord, ll_y_coord, cell_size, detect_prob, response,
                           effort, sppdist, xy_coord, xy_count, tr_x_coord, tr_y_coord,
                           sys_side_length);
                break;
            //Random rectangular quadrats survey design.
            case 'e':
            case 'E':
                //Calculates the length of rectangular quadrats.
                biased_length = ea_quadrat_area / biased_width;

                while (num_samples_count > 0)
                {
                    //Generates the angle for rectangular quadrat orientation.
                    biased_angle = randnum(0, 359);

                    seed(cell_size, survey_x_coord, ll_x_coord, survey_y_coord,
                         ll_y_coord);

                    rec_survey(num_samples, num_quadrats, biased_length, biased_width,
                               survey_x_coord, biased_angle, survey_y_coord, ll_x_coord,
                               ll_y_coord, cell_size, detect_prob, response, effort, sppdist,
                               xy_coord, xy_count, num_samples_count, tr_x_coord, tr_y_coord);
                }//end while (num_samples_count > 0)
                break;
        }//end switch (survey_design)

        create_files(prop_sampled, response, effort, num_samples, xy_coord, catcodes);
    }

    return 0;
}//end main


void seed(int cell_size, double& survey_x_coord, double ll_x_coord,
          double& survey_y_coord, double ll_y_coord)
//Generates a random seeding location within the entire study extent.
{
    survey_x_coord = ll_x_coord + double(randnum(1, (CELLS_PER_SIDE * cell_size)));
    survey_y_coord = ll_y_coord + double(randnum(1, (CELLS_PER_SIDE * cell_size)));
}//end seed


void sq_survey(int num_samples, int num_quadrats, double sq_side_length,
               double survey_x_coord, double survey_y_coord, double ll_x_coord,
               double ll_y_coord, int cell_size, double detect_prob, int response[],
               int effort[], int sppdist[], double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE],
               int& xy_count, int& num_samples_count, double tr_x_coord,
               double tr_y_coord)
//Algorithm generating a design for square quadrats.
//Setting proportion sampled to 1.0 and number of quadrats to 1, entire study extent
//will be sampled RANDOMLY.
// sample_x    = sample location x co-ordinate
// sample_y    = sample location y co-ordinate
// i           = incrementor
// random_num1 = distance to move from center of quadrat in x direction
// random_num2 = distance to move from center of quadrat in y direction
// direction   = used to determine in what direction to move from quadrat center
// detected    = tracks success of sampling effort
{
    double sample_x, sample_y;
    int i, random_num1, random_num2, direction;
    int detected = 0;

    //Generates the sample locations in each quadrat.
    for (i = 1; i <= (num_samples / num_quadrats); i++)
    {
        if (num_samples_count > 0)
        {
            random_num1 = randnum(0, int(sq_side_length / 2.0));
            random_num2 = randnum(0, int(sq_side_length / 2.0));
            direction = randnum(0, 1);

            samples(direction, sample_x, survey_x_coord, random_num1, sample_y,
                    survey_y_coord, random_num2);

            //If sampling location is within study extent.
            if ((sample_x > ll_x_coord) && (sample_y > ll_y_coord) &&
                (sample_x <= tr_x_coord) && (sample_y <= tr_y_coord))
            {
                num_samples_count--;

                observe_spp(sample_x, ll_x_coord, sample_y, ll_y_coord,
                            cell_size, detect_prob, response, effort,
                            sppdist, detected);

                rec_xy(xy_coord, sample_x, sample_y, xy_count);
            }//end if (within bounds)
        }//end if (num_samples_count > 0)
    }//end for_i
}//end void sq_survey


void strat_survey(int num_catcodes, int num_samples_count, double ll_x_coord,
                  int cell_size, double ll_y_coord, int num_samples,
                  double detect_prob, int response[], int effort[], 
                  int catcodes[], int sppdist[],
                  double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE],
                  int& xy_count)
//Algorithm dividing random sample locations equally between each class in
//a categorical variable.
// sample_x         = sample location x co-ordinate
// sample_y         = sample location y co-ordinate
// i                = incrementor
// cat_code_count   = tracks the number of samples drawn from each class
// sample_cell_num  = grid cell number of sample location
// sampled_cat_code = classification code of sample location
// stopper          = flag for successful increment
// detected         = flag for success of sampling effort
{
    double sample_x, sample_y;

    //cat_code_count - each code/count combination occupies 1 column, 
    // currently set = 3 codes
    int i, cat_code_count[2][3], sample_cell_num, sampled_cat_code;
    int stopper, detected = 0;

    //Sets array cells = 0.
    for (i = 0; i < num_catcodes; i++)
    {
        cat_code_count[0][i] = 0;
        cat_code_count[1][i] = 0;
    }//end for

    while (num_samples_count > 0)
    {
        //Generates random samples within study extent.
        sample_x = ll_x_coord + double(randnum(1, (CELLS_PER_SIDE * cell_size)));
        sample_y = ll_y_coord + double(randnum(1, (CELLS_PER_SIDE * cell_size)));

        sample_cell_num = cell_num(sample_x, ll_x_coord, cell_size, sample_y, 
                                  ll_y_coord);

        //Uses appropriate array cell for categorical code.
        sampled_cat_code = catcodes[(sample_cell_num - 1)];

        //Prevents multiple increments in one iteration (number samples per categorical class).
        stopper = 0;

        for (i = 0; i < num_catcodes; i++)
        {
            //Places count in approriate array column (one column for each categorical class).
            if (cat_code_count[0][i] == sampled_cat_code)
            {
                stopper = 1;

                //Counts the number of samples within a categorical class.
                if (cat_code_count[1][i] < (num_samples / num_catcodes))
                {
                    cat_code_count[1][i]++;
                    num_samples_count--;

                    observe_spp(sample_x, ll_x_coord, sample_y, ll_y_coord,
                                cell_size, detect_prob, response, effort,
                                sppdist, detected);

                    rec_xy(xy_coord, sample_x, sample_y, xy_count);
                }//end if
            }//end if
            //Fills a new column in array when an unsurveyed categorical class is encountered.
            else
            {
                if ((stopper == 0) && (cat_code_count[0][i] == 0))
                {
                    cat_code_count[0][i] = sampled_cat_code;
                    cat_code_count[1][i]++;
                    num_samples_count--;
                    stopper = 1;

                    observe_spp(sample_x, ll_x_coord, sample_y, ll_y_coord,
                                cell_size, detect_prob, response, effort,
                                sppdist, detected);

                    rec_xy(xy_coord, sample_x, sample_y, xy_count);
                }//end if
            }//end else
        }//end for
    }//end while (num_samples_count > 0)
}//end void strat_survey


void adapt_survey(int num_samples, int num_samples_count, 
                  int cell_size, double ll_x_coord, double ll_y_coord,
                  double tr_y_coord, double detect_prob, int response[], int effort[],
                  int sppdist[], double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE],
                  int& xy_count, double tr_x_coord)
//Algorithm generating an adaptive surveying procedure.
// survey_x_coord    = longitude co-ordinate for systematic sample location and grid cell 
//                      center
// survey_y_coord    = latitude co-ordinate for systematic sample location and grid cell 
//                      center
// temp_x            = holds longitudinal co-ordinates during hierarchical sampling
// temp_y            = holds latitudinal co-ordinates during hierarchical sampling
// detect_xy[][]     = co-ordinates of those grid cells where distribution was detected
// i, x, y           = incrementors
// detection_count   = tracks the number grid cells to sample around (hierarchical sampling)
// cell_numbers[]    = array for recording sampled grid cell numbers
// sample_cell_num   = grid cell number of sample location
// hold_cell_num     = variable for comparison against current sample_cell_num
// cell_num_count    = tracks the number of samples drawn
// center_cell_count = flag to indicate center cell during hierarchical sampling
// detected          = flag for success of sampling effort
// detect_xy_row     = tracks array row number (detect_xy[][]) in which to record 
//                      detection co-ordinates
// new_survey        = tracks array row number (detect_xy[][]) containing detection 
//                      co-ordinates to sample around

{
    double survey_x_coord, survey_y_coord, temp_x, temp_y, detect_xy[2][num_samples];
    int i, x, y, detection_count, cell_numbers[num_samples];
    int sample_cell_num, hold_cell_num, cell_num_count = 0, center_cell_count;
    int detected = 0, detect_xy_row, new_survey;

    //Sets arrays cells = 0.
    for (i = 0; i < num_samples; i++)
    {
        cell_numbers[i] = 0;
    }//end for

    while (num_samples_count > 0)
    {
        detection_count = 1;
        detect_xy_row = 0;
        new_survey = 0;

        seed(cell_size, survey_x_coord, ll_x_coord, survey_y_coord, ll_y_coord);

        sample_cell_num = cell_num(survey_x_coord, ll_x_coord, cell_size, survey_y_coord, 
                                   ll_y_coord);

        //This time through for surveying with replacement. Just sets arrays cells = 0,
        //prior to entering algorithm for adaptive hierarchy.
        for (i = 0; i < num_samples; i++)
        {
            cell_numbers[i] = 0;
        }//end for

        cell_num_count = 0;

        //Progresses through array (tracking those cells already surveyed) to determine
        //if current cell has been surveyed.
        i = 0;
        do
        {
            hold_cell_num = cell_numbers[i];
            i++;
        } while ((hold_cell_num != sample_cell_num) && (hold_cell_num != 0) && 
                 (i < num_samples));

        //If current cell has not been surveyed.
        if (hold_cell_num == 0)
        {
            num_samples_count--;
            cell_numbers[cell_num_count] = sample_cell_num;
            cell_num_count++;

            //Calculates the center coordinates for current cell.
            survey_x_coord = ll_x_coord + double((cell_size * ((sample_cell_num - 1)
                             % CELLS_PER_SIDE))) + (double(cell_size) * 0.5);
            survey_y_coord = tr_y_coord - (double(cell_size) * 
                             (ceil(double(sample_cell_num) / 
                             double(CELLS_PER_SIDE)))) + (double(cell_size) * 0.5);

            observe_spp(survey_x_coord, ll_x_coord, survey_y_coord, ll_y_coord,
                        cell_size, detect_prob, response, effort, sppdist, detected);

            rec_xy(xy_coord, survey_x_coord, survey_y_coord, xy_count);

            //If individual has been detected, sample all neighbors (n = 8).
            if (detected == 1)
            {
                do
                {
                    //Will signal when center cell (1st sampled) is reached in adaptive method.
                    center_cell_count = 0;

                    detection_count--;

                    //Repeated once for each row.
                    for (y = -1; y <= 1; y++)
                    {
                        //Repeated once for each column.
                        for (x = -1; x <= 1; x++)
                        {
                            center_cell_count++;

                            //Calculates cells' center x,y as progression made 
                            //through adaptive method.
                            temp_x = survey_x_coord + (cell_size * x);
                            temp_y = survey_y_coord + (cell_size * y);

                            //If x,y is within entire grid extent and not equal to center cell.
                            if ((temp_x > ll_x_coord) && (temp_y > ll_y_coord) &&
                                (temp_x <= tr_x_coord) && (temp_y <= tr_y_coord) &&
                                (center_cell_count != 5) && (num_samples_count > 0))
                            {
                                sample_cell_num = cell_num(temp_x, ll_x_coord, cell_size,
                                                          temp_y, ll_y_coord);

                                //Progresses through array (tracking those cells 
                                //already surveyed) 
                                //to determine if current cell has been surveyed.
                                i = 0;
                                do
                                {
                                    hold_cell_num = cell_numbers[i];
                                    i++;
                                } while ((hold_cell_num != sample_cell_num) && 
                                         (hold_cell_num != 0) && (i < num_samples));

                                //If current cell has not been surveyed.
                                if (hold_cell_num == 0)
                                {
                                    num_samples_count--;
                                    cell_numbers[cell_num_count] = sample_cell_num;
                                    cell_num_count++;

                                    observe_spp(temp_x, ll_x_coord, temp_y, ll_y_coord,
                                                cell_size, detect_prob, response, effort, 
                                                sppdist, detected);

                                    rec_xy(xy_coord, temp_x, temp_y, xy_count);

                                    //If individual is detected, record x,y coordinates 
                                    //of that cell.
                                    if (detected == 1)
                                    {
                                        detect_xy[0][detect_xy_row] = temp_x;
                                        detect_xy[1][detect_xy_row] = temp_y;
                                        detect_xy_row++;
                                        detection_count++;
                                    }//end if (detected == 1)
                                }//end if (hold_cell_num == 0)
                            }//end if (not center cell)
                        }//end for int x
                    }//end for int y

                    //Following each iteration, new center cell becomes first of those where 
                    //another individual has been detected.
                    survey_x_coord = detect_xy[0][new_survey];
                    survey_y_coord = detect_xy[1][new_survey];
                    new_survey++;
                } while (detection_count > 0);
            }//end if (detected == 1)
        }//end if (hold_cell_num == 0)
    }//end while (num_samples_count > 0)
}//end void adapt_survey


void sys_survey(int num_samples_count, int num_quadrats,
               double sq_side_length, double ll_x_coord,
               double ll_y_coord, int cell_size, double detect_prob,
               int response[], int effort[], int sppdist[],
               double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE],
               int& xy_count, double tr_x_coord, double tr_y_coord,
               double sys_side_length)
//Algorithm generating a design for sytematic square quadrats.
// survey_x_coord      = longitudinal co-ordinate for center of first quadrat
// survey_y_coord      = latitudinal co-ordinate for center of first quadrat
// sample_x            = sample location x co-ordinate
// sample_y            = sample location y co-ordinate
// temp_survey_x_coord = holds longitudinal co-ordinate for active quadrat
// temp_survey_y_coord = holds latitudinal co-ordinate for active quadrat
// random_num1         = distance to move from center of quadrat in x direction
// random_num2         = distance to move from center of quadrat in y direction
// direction           = used to determine in what direction to move from quadrat center
// detected            = tracks success of sampling effort
{
    double survey_x_coord, survey_y_coord, sample_x, sample_y;
    double temp_survey_x_coord, temp_survey_y_coord;
    int random_num1, random_num2, direction;
    int detected = 0;

    //Generates random quadrat location within the first systematic block.
    survey_x_coord = ll_x_coord + double(randnum(1, int(sys_side_length)));
    survey_y_coord = ll_y_coord + double(randnum(1, int(sys_side_length)));

    //Generates random samples within quadrats.
    random_num1 = randnum(0, int(sq_side_length / 2.0));
    random_num2 = randnum(0, int(sq_side_length / 2.0));
    direction = randnum(0, 1);

    samples(direction, sample_x, survey_x_coord, random_num1, sample_y, 
            survey_y_coord, random_num2);

    //If samples fall within study extent.
    if ((sample_x > ll_x_coord) && (sample_y > ll_y_coord) &&
        (sample_x <= tr_x_coord) && (sample_y <= tr_y_coord))
    {
        num_samples_count--;

        observe_spp(sample_x, ll_x_coord, sample_y, ll_y_coord,
                    cell_size, detect_prob, response, effort,
                    sppdist, detected);

        rec_xy(xy_coord, sample_x, sample_y, xy_count);
    }//end if (within bounds)

    //Preparation for moving to next systematic block.
    temp_survey_x_coord = survey_x_coord;
    temp_survey_y_coord = survey_y_coord;

    //Generates one sample in current quadrat.
    do
    {
        //Begins calculation for new quadrat in next systematic block.
        temp_survey_x_coord = temp_survey_x_coord + sys_side_length;

        //If new quadrat is entirely beyond eastern study boundary, begins new row of quadrats.
        if ((num_samples_count > 0) && (temp_survey_x_coord >=
            (tr_x_coord + (sq_side_length / 2.0))))
        {
            //Returns quadrat longitudinal co-ordinate to original location, one row north.
            temp_survey_x_coord = survey_x_coord;
            temp_survey_y_coord = temp_survey_y_coord + sys_side_length;

            if (temp_survey_y_coord >= (tr_y_coord + (sq_side_length / 2.0)))
            {
                temp_survey_y_coord = survey_y_coord;
            }
        }//end if

        //Generates random samples within quadrats.
        random_num1 = randnum(0, int(sq_side_length / 2.0));
        random_num2 = randnum(0, int(sq_side_length / 2.0));
        direction = randnum(0, 1);

        if (num_samples_count > 0)
        {
            samples(direction, sample_x, temp_survey_x_coord,
                   random_num1, sample_y, temp_survey_y_coord,
                   random_num2);

            //If samples fall within study extent.
            if ((sample_x > ll_x_coord) && (sample_y > ll_y_coord) &&
                (sample_x <= tr_x_coord) && (sample_y <= tr_y_coord))
            {
                num_samples_count--;

                observe_spp(sample_x, ll_x_coord, sample_y, ll_y_coord,
                            cell_size, detect_prob, response, effort,
                            sppdist, detected);

                rec_xy(xy_coord, sample_x, sample_y, xy_count);
            }//end if (within bounds)
        }//end if (num_samples_count > 0)
    } while (num_samples_count > 0);
}//end void sys_survey


void rec_survey(int num_samples, int num_quadrats, double biased_length,
                 double biased_width, double survey_x_coord, int biased_angle,
                 double survey_y_coord, double ll_x_coord, double ll_y_coord,
                 int cell_size, double detect_prob, int response[],
                 int effort[], int sppdist[],
                 double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE],
                 int& xy_count, int& num_samples_count, double tr_x_coord,
                 double tr_y_coord)
//Algorithm generating a design for random rectangular (i.e., biased) quadrats.
// sample_x    = sample location x co-ordinate
// sample_y    = sample location y co-ordinate
// i           = incrementor
// random_num1 = distance to move from center of quadrat in x direction
// random_num2 = distance to move from center of quadrat in y direction
// direction   = used to determine in what direction to move from quadrat center
// detected    = tracks success of sampling effort
{
    double sample_x, sample_y;
    int i, random_num1, random_num2, direction;
    int detected = 0;

    //Generates random samples within quadrats.
    for (i = 1; i <= (num_samples / num_quadrats); i++)
    {
        if (num_samples_count > 0)
        {
            random_num1 = randnum(0, int(biased_length / 2.0));
            random_num2 = randnum(0, int(biased_width / 2.0));
            direction = randnum(0, 1);

            //Location of sample in quadrat's length dimension generated (this if/else).
            if (direction == 0)
            {
                sample_x = survey_x_coord - (cos(double(biased_angle) * 2.0 * 
                           PI / 360.0) * double(random_num1));
                sample_y = survey_y_coord - (sin(double(biased_angle) * 2.0 *
                           PI / 360.0) * double(random_num1));
            }
            else
            {
                sample_x = survey_x_coord + (cos(double(biased_angle) * 2.0 * 
                           PI / 360.0) * double(random_num1));
                sample_y = survey_y_coord + (sin(double(biased_angle) * 2.0 *
                           PI / 360.0) * double(random_num1));
            }

            direction = randnum(0, 1);

            //Location of sample in quadrat's width dimension generated (this if/else).
            if (direction == 0)
            {
                sample_x -= (sin(double(biased_angle) * 2.0 * PI / 360.0) *
                             double(random_num2));
                sample_y += (cos(double(biased_angle) * 2.0 * PI / 360.0) * 
                             double(random_num2));
            }
            else
            {
                sample_x += (sin(double(biased_angle) * 2.0 * PI / 360.0) * 
                             double(random_num2));
                sample_y -= (cos(double(biased_angle) * 2.0 * PI / 360.0) * 
                             double(random_num2));
            }

            //If samples fall within study extent.
            if ((sample_x > ll_x_coord) && (sample_y > ll_y_coord) &&
                (sample_x <= tr_x_coord) && (sample_y <= tr_y_coord))
            {
                num_samples_count--; 

                observe_spp(sample_x, ll_x_coord, sample_y, ll_y_coord,
                            cell_size, detect_prob, response, effort,
                            sppdist, detected);

                rec_xy(xy_coord, sample_x, sample_y, xy_count);
            }//end if (within bounds)
        }//end if (num_samples_count > 0)
    }//end for_i
}//end void rec_survey


void samples(int direction, double& sample_x, double survey_x_coord,
             int random_num1, double& sample_y, double survey_y_coord,
             int random_num2)
//Algorithm generating random sample locations.
{
    //These if-else statements determine longitude, then latitude of sample.
    if (direction == 0)
    {
        sample_x = survey_x_coord - double(random_num1);
    }
    else
    {
        sample_x = survey_x_coord + double(random_num1);
    }

    direction = randnum(0, 1);

    if (direction == 0)
    {
        sample_y = survey_y_coord - double(random_num2);
    }
    else
    {
        sample_y = survey_y_coord + double(random_num2);
    }
}//end void samples


void observe_spp(double sample_x, double ll_x_coord, double sample_y, double ll_y_coord,
                 int cell_size, double detect_prob, int response[], int effort[],
                 int sppdist[], int& detected)
//Determines if sample location falls within the distribution
// sample_cell_num = grid cell number of sample location
{
    int sample_cell_num;

    detected = 0;

    sample_cell_num = cell_num(sample_x, ll_x_coord, cell_size, sample_y, ll_y_coord);

    //Uses appropriate species distribution array cell to determine presence/absence.
    if (sppdist[(sample_cell_num - 1)] == 1)
    {
        if ((double(randnum(0, 10000)) / double(10000)) <= detect_prob)
        {
            //Cells in which a detection is made are flagged with a 1 (needed by ALR model).
            response[sample_cell_num - 1] = 1;
            detected = 1;
        }
    }

    //Cells that are sampled flagged with a 0 (needed by ALR model).
    effort[sample_cell_num - 1] = 0;
}//end void observe_spp


void rec_xy(double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE], double sample_x,
            double sample_y, int& xy_count)
//Records sample longitude and latitude co-ordinates in an array.
{
    xy_coord[0][xy_count] = sample_x;
    xy_coord[1][xy_count] = sample_y;
    xy_count++;
}// end void rec_xy


void create_files(double prop_sampled, int response[], int effort[], 
                  int num_samples, double xy_coord[][CELLS_PER_SIDE * CELLS_PER_SIDE],
                  int catcodes[])
//Converts three detected absences to false-positives in categorical classes unoccupied 
//by the distribution.  Then writes contents of three arrays (response, effort, and [x,y])
//to three separate files.
// CatType = categorical code occupied by the distribution
// trash   = for discarding the first line of land cover type text files
// i,j     = incrementors
// CatCov   = array for categorical type
// Rand4LC = grid cell number to check for possible modification to false-positive
{
    int trash, i, j, Rand4LC;
    ofstream out_stream1, out_stream2, out_stream3;

    //Specific to categorical data with these codes, code checks if there are at least
    // 3 presences in each class.  If < 3 are found, will introduce the difference.
    if (CatType == 42)
    {
        //Track the number of false-positives in each categorical class.
        int count51 = 0, count71 = 0;

        for (i = 0; i < (CELLS_PER_SIDE * CELLS_PER_SIDE); i++)
        {
            if ((catcodes[i] == 51) && (effort[i] == 0) && (response[i] == 1))
            {
                count51++;
            }

            if ((catcodes[i] == 71) && (effort[i] == 0) && (response[i] == 1))
            {
                count71++;
            }
        }//end for i


        //Convert detected absences to false-positives in each class (if needed).
        while ((count51 < 3) || (count71 < 3))
        {
            Rand4LC = randnum(0, (CELLS_PER_SIDE * CELLS_PER_SIDE - 1));

            //If fewer than 3 false-positives have been introduced to 
            //sampled absence locations in LC = 51.
            if ((catcodes[Rand4LC] == 51) && (effort[Rand4LC] == 0) && 
                (response[Rand4LC] == 0) && (count51 < 3))
            {
                response[Rand4LC] = 1;
                count51++;
            }//end if = 51

            //If fewer than 3 false-positives have been introduced to 
            //sampled absence locations in LC = 71.
            if ((catcodes[Rand4LC] == 71) && (effort[Rand4LC] == 0) && 
                (response[Rand4LC] == 0) && (count71 < 3))
            {
                response[Rand4LC] = 1;
                count71++;
            }//end if = 71
        }
    }//end if CatType == 42


    if (CatType == 71)
    {
        //Track the number of false-positives in each categorical class.
        int count42 = 0, count51 = 0;

        for (i = 0; i < (CELLS_PER_SIDE * CELLS_PER_SIDE); i++)
        {
            if ((catcodes[i] == 42) && (effort[i] == 0) && (response[i] == 1))
            {
                count42++;
            }

            if ((catcodes[i] == 51) && (effort[i] == 0) && (response[i] == 1))
            {
                count51++;
            }
        }//end for i


        //Convert detected absences to false-positives in each class (if needed).
        while ((count42 < 3) || (count51 < 3))
        {
            Rand4LC = randnum(0, (CELLS_PER_SIDE * CELLS_PER_SIDE - 1));

            //If fewer than 3 false-positives have been introduced to 
            //sampled absence locations in LC = 42.
            if ((catcodes[Rand4LC] == 42) && (effort[Rand4LC] == 0) && 
                (response[Rand4LC] == 0) && (count42 < 3))
            {
                response[Rand4LC] = 1;
                count42++;
            }//end if = 42

            //If fewer than 3 false-positives have been introduced to 
            //sampled absence locations in LC = 51.
            if ((catcodes[Rand4LC] == 51) && (effort[Rand4LC] == 0) && 
                (response[Rand4LC] == 0) && (count51 < 3))
            {
                response[Rand4LC] = 1;
                count51++;
            }//end if = 51
        }
    }//end if CatType == 51)



    out_stream1.open("response.y");
    if (out_stream1.fail())
    {
        cout << "Output file (response.y) opening failed.\n";
        exit(1);
    }

    out_stream2.open("effort.g");
    if (out_stream2.fail())
    {
        cout << "Output file (effort.g) opening failed.\n";
        exit(1);
    }

    //Header line.
    out_stream1 << (CELLS_PER_SIDE * CELLS_PER_SIDE) << endl;
    out_stream2 << (CELLS_PER_SIDE * CELLS_PER_SIDE) << endl;

    //Rearranges data during file creation such that listing will begin in TL corner
    //and progress first down (rows) and then across (columns), required by ALR model.
    for (i = 1; i <= CELLS_PER_SIDE; i++)
    {
        for (j = 0; j < CELLS_PER_SIDE; j++)
        {
            out_stream1 << response[((j * CELLS_PER_SIDE) + i) - 1] << endl;
            out_stream2 << effort[((j * CELLS_PER_SIDE) + i) - 1] << endl;
        }
    }

    out_stream1.close();
    out_stream2.close();

    //For co-ordinate file.
    out_stream3.open("XY.txt");
    if (out_stream3.fail())
    {
        cout << "Output file (XY.txt) opening failed.\n";
        exit(1);
    }

    out_stream3.setf(ios::fixed);
    out_stream3.setf(ios::showpoint);
    out_stream3.precision(3);

    for (i = 1; i <= (num_samples); i++)
    {
        out_stream3 << i << "," << xy_coord[0][i - 1] << "," << xy_coord[1][i - 1] << endl;
    }

    out_stream3 << "end" << endl;
    out_stream3.close();
}//end void create_files


double quadrat_area(int cell_size, double prop_sampled, int num_quadrats)
//Computes the area of each quadrat.
{
    return (((double(CELLS_PER_SIDE) * double(cell_size)) *
             (double(CELLS_PER_SIDE) * double(cell_size)) * 
              prop_sampled) / double(num_quadrats));
}//end quadrat_area


int cell_num(double sample_x, double ll_x_coord, int cell_size, double sample_y,
             double ll_y_coord)
//Computes the grid cell number being sampled.
// sample_column_num = column number in which the sample falls
// sample_row_num    = row number in which the sample falls
{
    int sample_column_num, sample_row_num;

    //Calculate the grid cell row and column of the sample.
    sample_column_num = int(ceil((sample_x - ll_x_coord) / double(cell_size)));
    sample_row_num = int(ceil((sample_y - ll_y_coord) / double(cell_size)));

    return (((CELLS_PER_SIDE - sample_row_num) * CELLS_PER_SIDE) + sample_column_num);
}//end void cell_num


int randnum(int lower, int upper)
//Generates a random number between lower and upper arguments, based on clock.
// randomNumber = randomly generated number
{
    int randomNumber;
    static short seed = 0;
    time_t t;

    if (seed == 0)
    {
        srand((unsigned) time(&t));
        seed = 1;
    }
    randomNumber = rand() % (upper - lower + 1) + lower;
    return randomNumber;
}//end int randnum
